@@ -143,6 +143,7 @@ class SuggestionButton ( Gtk.Button ):
         Align1.show()
         self.add( Align1 )
         self.show()
+        self.connect( "enter-notify-event", self.onEnter )
         self.connect( "focus-in-event", self.onFocusIn )
         self.connect( "focus-out-event", self.onFocusOut )
 
@@ -155,6 +156,9 @@ class SuggestionButton ( Gtk.Button ):
     def set_icon_size (self, size):
         self.image.set_pixel_size( size )
 
+    def onEnter( self, widget, event ):
+        self.grab_focus()
+
     def onFocusIn( self, widget, event ):
         self.set_state_flags( Gtk.StateFlags.PRELIGHT, False )
 
@@ -246,6 +250,7 @@ class pluginclass( object ):
             self.settings.bindGSettingsEntryToVar( "bool", "do-not-filter", self, "donotfilterapps" )
             self.settings.bindGSettingsEntryToVar( "string", "search-command", self, "searchtool" )
             self.settings.bindGSettingsEntryToVar( "int", "default-tab", self, "defaultTab" )
+            self.settings.bindGSettingsEntryToVar( "bool", "always-show-search", self, "alwaysshowsearch" )
             self.settings.bindGSettingsEntryToVar( "bool", "enable-ddg", self, "enableddg" )
             self.settings.bindGSettingsEntryToVar( "bool", "enable-google", self, "enablegoogle" )
             self.settings.bindGSettingsEntryToVar( "bool", "enable-wikipedia", self, "enablewikipedia" )
@@ -438,6 +443,7 @@ class pluginclass( object ):
         self.categoryhoverdelay = self.settings.get( "int", "category-hover-delay")
         self.showapplicationcomments = self.settings.get( "bool", "show-application-comments")
         self.rememberFilter = self.settings.get( "bool", "remember-filter")
+        self.alwaysshowsearch = self.settings.get( "bool", "always-show-search")
         self.enableddg = self.settings.get( "bool", "enable-ddg")
         self.enablegoogle = self.settings.get( "bool", "enable-google")
         self.enablewikipedia = self.settings.get( "bool", "enable-wikipedia")
@@ -515,9 +521,11 @@ class pluginclass( object ):
         self.searchEntry.connect( "changed", self.Filter )
         self.searchEntry.connect( "activate", self.Search )
         self.showAllAppsButton.connect( "clicked", lambda widget: self.changeTab( 1 ) )
+        self.showAllAppsButton.connect( "enter-notify-event", self.onEnter )
         self.showAllAppsButton.connect( "focus-in-event", self.onFocusIn )
         self.showAllAppsButton.connect( "focus-out-event", self.onFocusOut )
         self.showFavoritesButton.connect( "clicked", lambda widget: self.changeTab( 0 ) )
+        self.showFavoritesButton.connect( "enter-notify-event", self.onEnter )
         self.showFavoritesButton.connect( "focus-in-event", self.onFocusIn )
         self.showFavoritesButton.connect( "focus-out-event", self.onFocusOut )
         self.buildButtonList()
@@ -531,6 +539,9 @@ class pluginclass( object ):
             self.mateMenuWin.stopHiding()
         return False
 
+    def onEnter( self, widget, event ):
+        widget.grab_focus()
+
     def onFocusIn( self, widget, event ):
         widget.set_state_flags( Gtk.StateFlags.PRELIGHT, False )
 
@@ -538,14 +549,20 @@ class pluginclass( object ):
         widget.unset_state_flags( Gtk.StateFlags.PRELIGHT )
 
     def focusSearchEntry( self, clear = True ):
-        # grab_focus() does select all text,
-        # restoring the original selection is somehow broken, so just select the end
-        # of the existing text, that's the most likely candidate anyhow
-        self.searchEntry.grab_focus()
-        if self.rememberFilter or not clear:
-            self.searchEntry.set_position(-1)
-        else:
-            self.searchEntry.set_text("")
+        def doFocusSearchEntry():
+            # grab_focus() does select all text,
+            # restoring the original selection is somehow broken, so just select the end
+            # of the existing text, that's the most likely candidate anyhow
+            self.searchEntry.grab_focus()
+            if self.rememberFilter or not clear:
+                self.searchEntry.set_position(-1)
+            else:
+                self.searchEntry.set_text("")
+
+        # Focus after a small timeout so that the pointer doesn't steal focus
+        # from the search entry if its position happens to be within the menu
+        # window.
+        GLib.timeout_add(100, doFocusSearchEntry)
 
     def buildButtonList( self ):
         if self.buildingButtonList:
@@ -572,10 +589,10 @@ class pluginclass( object ):
             GLib.source_remove( self.filterTimer )
             self.filterTimer = None
 
-    def add_search_suggestions(self, text):
+    def add_search_suggestions(self, text, already_focused = False):
 
         text = "<b>%s</b>" % text
-        focused = False
+        focused = already_focused
 
         if self.enableddg:
             suggestionButton = SuggestionButton("list-add", self.iconSize, "")
@@ -690,8 +707,6 @@ class pluginclass( object ):
                         self.current_suggestion = None
                         self.current_results = []
                 else:
-                    self.current_suggestion = None
-                    self.current_results = []
                     # Sort applications by relevance, and alphabetical within that
                     shownList = sorted(shownList, key=lambda app: app.appName)
                     shownList = sorted(shownList, key=lambda app: app.relevance, reverse=True)
@@ -700,8 +715,14 @@ class pluginclass( object ):
                         self.applicationsBox.add(i)
                         if not focused:
                             # Grab focus of the first app shown
-                            i.grab_focus()
+                            GLib.timeout_add(20, i.grab_focus)
                             focused = True
+                    if self.alwaysshowsearch:
+                        self.add_search_suggestions(text, focused)
+                        self.current_suggestion = text
+                    else:
+                        self.current_suggestion = None
+                        self.current_results = []
 
                 for i in self.categoriesBox.get_children():
                     i.set_relief( Gtk.ReliefStyle.NONE )
@@ -878,6 +899,13 @@ class pluginclass( object ):
     def searchPopup( self, widget=None, event=None ):
         menu = Gtk.Menu()
 
+        menuItem = Gtk.ImageMenuItem(label=_("Search DuckDuckGo"))
+        img = Gtk.Image()
+        img.set_from_file('/usr/share/mate-menu/icons/search_engines/ddg.ico')
+        menuItem.set_image(img)
+        menuItem.connect("activate", self.search_ddg)
+        menu.append(menuItem)
+
         menuItem = Gtk.ImageMenuItem(label=_("Search Google"))
         img = Gtk.Image()
         img.set_from_file('/usr/share/mate-menu/icons/search_engines/google.ico')
